; ========================================================================== ;
;                               = YM Player =                                ;
;                     Intellivision ROM by A.Chevallier                      ;
; -------------------------------------------------------------------------- ;
;                        arnauld.chevallier@gmail.com                        ;
; ========================================================================== ;

; -------------------------------------------------------------------------- ;
; 16-bit ROM                                                                 ;
; -------------------------------------------------------------------------- ;
            ROMW        16

; -------------------------------------------------------------------------- ;
; Common constants and macros                                                ;
; -------------------------------------------------------------------------- ;
            INCLUDE     "cst.asm"
            INCLUDE     "player.mac"

N_STREAM    EQU         10                      ; number of data streams
WIN_SIZ     EQU         1024                    ; size of unpacking window

UBLK_W      EQU         5                       ; bits used to store unpacked block size
REFP_W      EQU         10                      ; bits used to store position of reference
REFL_W      EQU         8                       ; bits used to store length of reference

; -------------------------------------------------------------------------- ;
; Scratch RAM ($0102 - $01EF)                                                ;
; -------------------------------------------------------------------------- ;
            ORG         $0102, $0102, "-RWBN"

FRAME       RMB         1                       ; frame counter
FRAMESKIP   RMB         1                       ; frame skip counter

; -------------------------------------------------------------------------- ;
; System RAM ($02F0 - $035F)                                                 ;
; -------------------------------------------------------------------------- ;
            ORG         $02F0, $02F0, "-RWBN"

STACK       RMB         21                      ; stack

VBTASK      RMB         1                       ; vblank task

M_FRAME     RMB         1                       ; music frame
TICK        RMB         1                       ; time variable
SECOND      RMB         1
MINUTE      RMB         1

PSG_TASK    RMB         1                       ; PSG update VBlank task

P_A         RMB         1                       ; backup of channel periods
P_B         RMB         1
P_C         RMB         1
P_E         RMB         1

RD_PTR      RMB         1                       ; song read pointer
P_VAR       RMB         1                       ; pointer to unpacking variables
W_MIN       RMB         1                       ; min. address of current unpacking window
W_MAX       RMB         1                       ; max. address of current unpacking window

PACKED0     EQU         $
P_WRD0      RMB         1                       ; +0 : last word read from packed data
P_BIT0      RMB         1                       ; +1 : number of bits available in P_WRD
P_PTR0      RMB         1                       ; +2 : packed data pointer
P_WIDTH0    RMB         1                       ; +3 : width of packed data
U_WR0       RMB         1                       ; +4 : unpacked data pointer
END0        EQU         $

P_V_SIZ     EQU         END0 - PACKED0

PACKED1     RMB         P_V_SIZ
PACKED2     RMB         P_V_SIZ
PACKED3     RMB         P_V_SIZ
PACKED4     RMB         P_V_SIZ
PACKED5     RMB         P_V_SIZ
PACKED6     RMB         P_V_SIZ
PACKED7     RMB         P_V_SIZ
PACKED8     RMB         P_V_SIZ
PACKED9     RMB         P_V_SIZ
P_END       EQU         $

KEY_ON      RMB         1                       ; 'keypad on' flag

MUTE_A      RMB         1                       ; muted channels
MUTE_B      RMB         1
MUTE_C      RMB         1

; -------------------------------------------------------------------------- ;
; Extra RAM                                                                  ;
; -------------------------------------------------------------------------- ;
            ORG         $9000, $9000, "=RW"

UNPK0       RMB         WIN_SIZ                 ; period / channel A
UNPK1       RMB         WIN_SIZ                 ; volume / channel A
UNPK2       RMB         WIN_SIZ                 ; period / channel B
UNPK3       RMB         WIN_SIZ                 ; volume / channel B
UNPK4       RMB         WIN_SIZ                 ; period / channel C
UNPK5       RMB         WIN_SIZ                 ; volume / channel C
UNPK6       RMB         WIN_SIZ                 ; envelope period
UNPK7       RMB         WIN_SIZ                 ; envelope type
UNPK8       RMB         WIN_SIZ                 ; noise period
UNPK9       RMB         WIN_SIZ                 ; mixer
U_END       EQU         $

; -------------------------------------------------------------------------- ;
; ROM header                                                                 ;
; -------------------------------------------------------------------------- ;
            ORG         $5000

rom_header  PROC

            BIDECLE     @@null                  ; MOB definition
            BIDECLE     @@null                  ; process table
            BIDECLE     @@null                  ; main entry point
            BIDECLE     @@null                  ; bg picture base
            BIDECLE     @@gram_pic              ; GRAM picture base
            BIDECLE     @@title                 ; date & title
            DECLE       $03C0                   ; Joe told me to put that ;-)

@@null:     BIDECLE     $0000

            DECLE       C_BLK, C_BLK            ; initial color stack
            DECLE       C_BLK, C_BLK

            DECLE       C_BLK                   ; border color

@@gram_pic: DECLE       $0001

@@title:    DECLE       2010 - 1900
            STRING      "YM Player", 0

            ENDP

; -------------------------------------------------------------------------- ;
; Initialization                                                             ;
; -------------------------------------------------------------------------- ;
start       PROC

            CALL        clear                   ; clear system RAM
            DECLE       $2F0, $70

            MVII        #STACK,     R6          ; init. stack

            MVII        #RTN_INT,   R0          ; init. VBlank
            MVO         R0,         VBTASK

            MVII        #vblank,    R0
            MVO         R0,         VBLK0
            SWAP        R0
            MVO         R0,         VBLK1

            CALL        clear                   ; clear screen
            DECLE       BT, 240

            JSRE        R5,         wait_vbl

            CLRR        R0
            MVO         R0,         BORDER      ; black border
            MVO         R0,         HDELAY      ; no horizontal delay
            MVO         R0,         VDELAY      ; no vertical delay
            MVO         R0,         B_EXT       ; no border extension

            MVO         R0,         CSTACK      ; color stack : black & green
            MVO         R0,         CSTACK+2
            MVII        #C_GRN,     R0
            MVO         R0,         CSTACK+1

            MVI         $0021,      R0          ; colorstack mode

            MVII        #$00AA,     R1          ; pattern used for system type detection
            MOVR        R1,         R2          ; max. number of iterations
            MVII        #vb.psg60,  R3          ; default value : 60Hz Inty

            CALL        wait_vbl                ; returns ~296 cycles after interrupt

@@bus_isol: DECR        R2                      ; (6) if all iterations were done w/out bus isolation,
            BEQ         @@sys_ok                ; (7) assume we're running under a (bad) 60Hz emulator

            MVO         R1,         GRAM        ; (11) write pattern in GRAM
            MVI         GRAM,       R0          ; (10) read it back
            ANDI        #$FF,       R0          ; (8)
            CMPR        R1,         R0          ; (6) same value ?
            BEQ         @@bus_isol              ; (9) total = 57 cycles per iteration

            CMPI        #96,        R2          ; 60Hz Inty ? ...
            BGT         @@sys_ok

            MVII        #vb.psg50,  R3          ; ... no : 50Hz Inty

@@sys_ok:   MVO         R3,         PSG_TASK

            CALL        wait_vbl                ; load GRAM cards
            MVII        #gfx,       R4
            MVII        #GRAM,      R5
            MVII        #40,        R1

@@load:     MVI@        R4,         R0
            MVO@        R0,         R5
            SWAP        R0
            MVO@        R0,         R5
            DECR        R1
            BNEQ        @@load

            CALL        init_unpk               ; init. unpacking process

            MVII        #$069D,     R0
            MVII        #20,        R1
            MVII        #BT+100,    R4

@@fill0:    MVO@        R0,         R4
            DECR        R1
            BNEQ        @@fill0

            MVII        #20,        R1
            MVII        #BT+180,    R4

@@fill1:    MVO@        R0,         R4
            DECR        R1
            BNEQ        @@fill1

            CALL        grom_text_p             ; draw screen
            DECLE       C_WHT, txt.1, BT+2
            CALL        grom_text_p.ptr
            DECLE       txt.2, BT+60
            CALL        grom_text_p.ptr
            DECLE       txt.3, BT+120
            CALL        grom_text_p.ptr
            DECLE       txt.4, BT+200

            CALL        grom_text_p
            DECLE       C_YEL, data.name, BT+40
            CALL        grom_text_p.ptr
            DECLE       data.author, BT+80

            MVI         data.info,  R1
            CALL        hex2dec
            DECLE       hex2dec.5
            DECLE       C_YEL + 16*8, BT+215

            MVI         data.info+1,R1
            CALL        hex2dec
            DECLE       hex2dec.2
            DECLE       C_YEL + 16*8, BT+235

            MVI         data.info+2,R1
            CALL        hex2dec
            DECLE       hex2dec.2
            DECLE       C_YEL + 16*8, BT+238

            MVII        #$2000,     R0
            MVO         R0,         BT
            XOR         BT+20,      R0
            MVO         R0,         BT+20

            MVII        #50,        R0          ; init. time
            MVO         R0,         TICK
            MVII        #-1,        R0
            MVO         R0,         SECOND

            MVI         PSG_TASK,   R0          ; start music
            MVO         R0,         VBTASK

            ENDP

; -------------------------------------------------------------------------- ;
; Main loop                                                                  ;
; -------------------------------------------------------------------------- ;
main        PROC

@@loop:     CALL        unpk_data

            B           @@loop

            ENDP

; -------------------------------------------------------------------------- ;
; [ unpk_data ]                                                              ;
; -------------------------------------------------------------------------- ;
; Unpack data                                                                ;
; -------------------------------------------------------------------------- ;
unpk_data   PROC
            PSHR        R5

            MVII        #PACKED0,   R4
            MVII        #UNPK0,     R5

@@loop:     MVO         R4,         P_VAR
            ADDI        #4,         R4
            MVI@        R4,         R0          ; R0 = U_WRx

            SUBR        R5,         R0          ; enough data available ? ...
            SUB         RD_PTR,     R0
            BPL         @@test

            ADDI        #WIN_SIZ,   R0

@@test:     CMPI        #16,        R0
            BGE         @@next_a

            MVO         R5,         W_MIN
            ADDI        #WIN_SIZ,   R5
            MVO         R5,         W_MAX

            CALL        unpk_bit                ; ... no : get compression method
            DECLE       1

            TSTR        R3                      ; backward reference ? ...
            BEQ         @@blk

            CALL        unpk_bit                ; ... yes : get position of reference
            DECLE       REFP_W

            PSHR        R3

            CALL        unpk_bit                ; get length of reference
            DECLE       REFL_W

            MVI         P_VAR,      R4
            ADDI        #4,         R4
            MVI@        R4,         R5          ; R4 = R5 = U_WRx
            MOVR        R5,         R4
            SUB@        R6,         R4          ; R4 = pointer to reference
            DECR        R4

            CMP         W_MIN,      R4
            BGE         @@copy

            ADDI        #WIN_SIZ,   R4

@@copy:     CMP         W_MAX,      R4          ; copy reference
            BLT         @@copy_r

            SUBI        #WIN_SIZ,   R4

@@copy_r:   MVI@        R4,         R0

            CMP         W_MAX,      R5
            BLT         @@copy_w

            SUBI        #WIN_SIZ,   R5

@@copy_w:   MVO@        R0,         R5

            DECR        R3
            BPL         @@copy

            MVI         P_VAR,      R4          ; save updated U_WRx
            ADDI        #4,         R4
            MVO@        R5,         R4

            B           @@next

@@blk:      CALL        unpk_bit                ; new block : read size
            DECLE       UBLK_W

            MVI         P_VAR,      R5
            ADDI        #3,         R5
            MVI@        R5,         R0          ; R0 = width of packed data
            MVI@        R5,         R4          ; R4 = U_WRx

@@save:     PSHR        R0
            PSHR        R3
            PSHR        R4

            CALL        unpk_bit.0              ; unpack data
            PULR        R4                      ; save it into buffer
            CMP         W_MAX,      R4
            BLT         @@save_w

            SUBI        #WIN_SIZ,   R4

@@save_w:   MVO@        R3,         R4

            PULR        R3
            PULR        R0
            DECR        R3
            BPL         @@save

            MVI         P_VAR,      R5
            ADDI        #4,         R5
            MVO@        R4,         R5          ; save updated U_WRx

@@next:     MVI         W_MIN,      R5          ; next stream
@@next_a:   ADDI        #WIN_SIZ,   R5

            MVI         P_VAR,      R4
            ADDI        #P_V_SIZ,   R4
            CMPI        #P_END,     R4
            BLT         @@loop

            PULR        R7
            ENDP

; -------------------------------------------------------------------------- ;
; [ init_unpk ]                                                              ;
; -------------------------------------------------------------------------- ;
; Initialize unpacking process                                               ;
; -------------------------------------------------------------------------- ;
init_unpk   PROC
            PSHR        R5

            CALL        clear
            DECLE       UNPK0, N_STREAM * WIN_SIZ

            MVII        #PACKED0,   R4
            MVII        #@@ini,     R5
            MVII        #N_STREAM,  R1

            MVII        #UNPK0,     R2

            CLRR        R0
            MVO         R0,         RD_PTR

@@fill:     MVO@        R0,         R4          ; P_WRDx
            MVO@        R0,         R4          ; P_BITx
            MVI@        R5,         R3
            MVO@        R3,         R4          ; P_PTRx
            MVI@        R5,         R3
            MVO@        R3,         R4          ; P_WIDTHx
            MVO@        R2,         R4          ; U_WRx
            ADDI        #WIN_SIZ,   R2

            DECR        R1
            BNEQ        @@fill

            MVII        #16,        R1          ; initiate unpacking process

@@unpk:     PSHR        R1
            CALL        unpk_data     
            PULR        R1
            DECR        R1
            BNEQ        @@unpk

            PULR        R7

@@ini:      DECLE       data.p0, 12
            DECLE       data.v0, 05
            DECLE       data.p1, 12
            DECLE       data.v1, 05
            DECLE       data.p2, 12
            DECLE       data.v2, 05
            DECLE       data.ep, 16
            DECLE       data.et, 04
            DECLE       data.np, 05
            DECLE       data.mx, 06

            ENDP

; -------------------------------------------------------------------------- ;
; [ unpk_bit ]                                                               ;
; -------------------------------------------------------------------------- ;
; Unpack bits from packed data stream                                        ;
; -------------------------------------------------------------------------- ;
unpk_bit    PROC

            MVI@        R5,         R0          ; R0 = number of bits to read

@@0:        PSHR        R5

            MVI         P_VAR,      R5
            MVI@        R5,         R1          ; R1 = P_BIT = number of bits available in P_WRD
            MVI@        R5,         R2          ; R2 = P_WRD = current packed word
            CLRR        R3                      ; R3 = result

@@rd_bit:   DECR        R1                      ; at least one bit available ?
            BMI         @@rd_word

@@bit:      SARC        R2                      ; get bit from P_WRD
            RLC         R3                      ; set it in result

            DECR        R0                      ; more bits to unpack ?
            BNEQ        @@rd_bit

            SUBI        #2,         R5
            MVO@        R1,         R5          ; save updated P_BIT
            MVO@        R2,         R5          ; save updated P_WRD

            PULR        R7

@@rd_word:  MVI@        R5,         R4          ; read new word from packed data stream
            MVI@        R4,         R2
            DECR        R5
            MVO@        R4,         R5
            DECR        R5
            MVII        #15,        R1
            B           @@bit

            ENDP

; -------------------------------------------------------------------------- ;
; [ vb ]                                                                     ;
; -------------------------------------------------------------------------- ;
; VBlank tasks                                                               ;
; -------------------------------------------------------------------------- ;
vb          PROC

@@psg60:    MVI         FRAMESKIP,  R0          ; update PSG now ? ...
            INCR        R0
            MVO         R0,         FRAMESKIP
            CMPI        #5,         R0
            BLE         @@psg50

            CLRR        R0                      ; ... no : skip this frame
            MVO         R0,         FRAMESKIP

            B           @@hc

@@psg50:    MVI         M_FRAME,    R0          ; end of song ? ...
            CMP         data.info,  R0
            BLT         @@upd_psg

            MVII        #%111111,   R0          ; ... yes : mute everything
            MVO         R0,         PSG.MIX

            CALL        clear                   ; clear VU-meters
            DECLE       BT+140, 20

            B           @@hc

@@upd_psg:  MVI         RD_PTR,     R3          ; update PSG
            INCR        R3
            MVO         R3,         RD_PTR

            ADDI        #UNPK0-1,   R3          ; period / channel A
            MVI@        R3,         R0

            MVO         R0,         P_A
            MVO         R0,         PSG.PL_A
            SWAP        R0
            MVO         R0,         PSG.PH_A

            ADDI        #WIN_SIZ,   R3          ; volume / channel A

            MVI         MUTE_A,     R0
            TSTR        R0
            BEQ         @@a_on

            CLRR        R0
            B           @@vol_a

@@a_on:     MVI@        R3,         R0
            CMPI        #$10,       R0
            BLT         @@vol_a

            MVII        #$30,       R0

@@vol_a:    MVO         R0,         PSG.VOL_A

            ADDI        #WIN_SIZ,   R3          ; period / channel B
            MVI@        R3,         R0

            MVO         R0,         P_B
            MVO         R0,         PSG.PL_B
            SWAP        R0
            MVO         R0,         PSG.PH_B

            ADDI        #WIN_SIZ,   R3          ; volume / channel B

            MVI         MUTE_B,     R0
            TSTR        R0
            BEQ         @@b_on

            CLRR        R0
            B           @@vol_b

@@b_on:     MVI@        R3,         R0
            CMPI        #$10,       R0
            BLT         @@vol_b

            MVII        #$30,       R0

@@vol_b:    MVO         R0,         PSG.VOL_B

            ADDI        #WIN_SIZ,   R3          ; period / channel C
            MVI@        R3,         R0

            MVO         R0,         P_C
            MVO         R0,         PSG.PL_C
            SWAP        R0
            MVO         R0,         PSG.PH_C

            ADDI        #WIN_SIZ,   R3          ; volume / channel C

            MVI         MUTE_C,     R0
            TSTR        R0
            BEQ         @@c_on

            CLRR        R0
            B           @@vol_c

@@c_on:     MVI@        R3,         R0
            CMPI        #$10,       R0
            BLT         @@vol_c

            MVII        #$30,       R0

@@vol_c:    MVO         R0,         PSG.VOL_C

            ADDI        #WIN_SIZ,   R3          ; envelope period
            MVI@        R3,         R0

            MVO         R0,         P_E
            MVO         R0,         PSG.PL_E
            SWAP        R0
            MVO         R0,         PSG.PH_E

            ADDI        #WIN_SIZ,   R3          ; envelope type
            MVI@        R3,         R0
            TSTR        R0
            BEQ         @@np

            MVO         R0,         PSG.ENV

@@np:       ADDI        #WIN_SIZ,   R3          ; noise period
            MVI@        R3,         R0

            MVO         R0,         PSG.NP

            ADDI        #WIN_SIZ,   R3          ; mixer
            MVI@        R3,         R0

            MVO         R0,         PSG.MIX

            CALL        draw_hex                ; draw mixer
            DECLE       2, BT+139

            MVI         PSG.NP,     R0          ; draw noise period
            CALL        draw_hex
            DECLE       2, BT+133

            MVI         PSG.ENV,    R0          ; draw envelope type
            CALL        draw_hex
            DECLE       1, BT+122

            MVI         P_E,        R0          ; draw envelope period
            CALL        draw_hex
            DECLE       4, BT+127

            MVI         P_A,        R0          ; draw channel A period
            CALL        draw_hex
            DECLE       3, BT+162

            MVI         PSG.VOL_A,  R0          ; draw channel A volume
            CALL        draw_hex
            DECLE       2, BT+165

            MVI         P_B,        R0          ; draw channel B period
            CALL        draw_hex
            DECLE       3, BT+169

            MVI         PSG.VOL_B,  R0          ; draw channel B  volume
            CALL        draw_hex
            DECLE       2, BT+172

            MVI         P_C,        R0          ; draw channel C period
            CALL        draw_hex
            DECLE       3, BT+176

            MVI         PSG.VOL_C,  R0          ; draw channel C volume
            CALL        draw_hex
            DECLE       2, BT+179

            CALL        vu_meter                ; draw channel A VU-meter
            DECLE       MUTE_A, PSG.VOL_A
            DECLE       BT+140

            CALL        vu_meter                ; draw channel B VU-meter
            DECLE       MUTE_B, PSG.VOL_B
            DECLE       BT+147

            CALL        vu_meter                ; draw channel C VU-meter
            DECLE       MUTE_C, PSG.VOL_C
            DECLE       BT+154

            MVI         M_FRAME,    R1          ; music frame++
            INCR        R1
            MVO         R1,         M_FRAME

            CALL        hex2dec
            DECLE       hex2dec.5
            DECLE       C_YEL + 16*8, BT+207

            MVI         TICK,       R1          ; tick++
            INCR        R1
            MVO         R1,         TICK

            CMPI        #50,        R1
            BLT         @@wrap

            SUBI        #50,        R1
            MVO         R1,         TICK

            MVI         SECOND,     R1          ; second++
            INCR        R1

            CMPI        #60,        R1
            BNEQ        @@upd_time

            MVI         MINUTE,     R0          ; minute++
            INCR        R0
            MVO         R0,         MINUTE

            CLRR        R1

@@upd_time: MVO         R1,         SECOND      ; draw time

            CALL        hex2dec
            DECLE       hex2dec.2
            DECLE       C_YEL + 16*8, BT+230

            MVI         MINUTE,     R1

            CALL        hex2dec
            DECLE       hex2dec.2
            DECLE       C_YEL + 16*8, BT+227

@@wrap:     MVI         RD_PTR,     R0          ; wrap ? ...
            CMPI        #WIN_SIZ,   R0
            BLT         RTN_INT

            CLRR        R0                      ; ... yes
            MVO         R0,         RD_PTR

@@hc:       MVI         PSG.IO_R,   R0          ; test hand controllers
            AND         PSG.IO_L,   R0
            XORI        #$FF,       R0
            BEQ         @@key_off

            MVI         KEY_ON,     R1
            TSTR        R1
            BNEQ        RTN_INT
            B           @@chk_hc

@@key_off:  MVO         R0,         KEY_ON
            B           RTN_INT

@@chk_hc:   MVO         R0,         KEY_ON

            CMPI        #$81,       R0
            BEQ         @@mute_a

            CMPI        #$41,       R0
            BEQ         @@mute_b

            CMPI        #$21,       R0
            BNEQ        RTN_INT

            MVI         MUTE_C,     R0          ; toggle channel C on/off
            COMR        R0
            MVO         R0,         MUTE_C
            B           RTN_INT

@@mute_a:   MVI         MUTE_A,     R0          ; toggle channel A on/off
            COMR        R0
            MVO         R0,         MUTE_A
            B           RTN_INT

@@mute_b:   MVI         MUTE_B,     R0          ; toggle channel B on/off
            COMR        R0
            MVO         R0,         MUTE_B
            B           RTN_INT

            ENDP

; -------------------------------------------------------------------------- ;
; [ vu_meter ]                                                               ;
; -------------------------------------------------------------------------- ;
; Draw VU-meter                                                              ;
; -------------------------------------------------------------------------- ;
vu_meter    PROC

            MVI@        R5,         R2          ; R2 = mute flag
            MVI@        R2,         R2
            MVI@        R5,         R1          ; R1 = PSG volume register
            MVI@        R5,         R4          ; R4 = backtab
            TSTR        R2
            BEQ         @@upd

            MVII        #$06C,      R0          ; '-'
            MVO@        R0,         R4
            MVII        #$26C,      R1          ; 'm'
            MVO@        R1,         R4
            MVII        #$2AC,      R1          ; 'u'
            MVO@        R1,         R4
            MVII        #$2A4,      R1          ; 't'
            MVO@        R1,         R4
            MVII        #$22C,      R1          ; 'e'
            MVO@        R1,         R4
            MVO@        R0,         R4

            JR          R5

@@upd:      MVI@        R1,         R1
            MOVR        R1,         R2
            ANDI        #$10,       R2
            BEQ         @@read

            MVII        #8,         R1

@@read:     ADDI        #@@conv,    R1
            MVI@        R1,         R1

            MOVR        R1,         R2
            SLR         R1,         2
            SLR         R1,         2
            ANDI        #$F,        R2
            SLL         R2,         2
            ADDI        #$0806,     R2
            MVO@        R2,         R4
            ADDI        #$28,       R2
            ADDI        #4,         R4
            MVO@        R2,         R4

            MOVR        R1,         R2
            SLR         R1,         2
            SLR         R1,         2
            ANDI        #$F,        R2
            SLL         R2,         2
            ADDI        #$1806+$28, R2
            SUBI        #2,         R4
            MVO@        R2,         R4
            SUBI        #$28,       R2
            SUBI        #4,         R4
            MVO@        R2,         R4

            ANDI        #$F,        R1
            SLL         R1,         2
            ADDI        #$0805,     R1
            MVO@        R1,         R4
            ADDI        #$28,       R1
            MVO@        R1,         R4

            JR          R5

@@conv:     DECLE       $000, $200, $200, $400, $400, $600, $600, $800
            DECLE       $820, $840, $860, $880, $882, $884, $886, $888

            ENDP

; -------------------------------------------------------------------------- ;
; Texts                                                                      ;
; -------------------------------------------------------------------------- ;
txt         PROC

@@1:        STRING16    "YM-Player by A.C  Track"
@@2:        STRING16    "Author"
@@3:        STRING16    "E: -     NP:   MX:"
@@4:        STRING16    "Frame        /      Time     :   /   :"

            ENDP

; -------------------------------------------------------------------------- ;
; Graphics                                                                   ;
; -------------------------------------------------------------------------- ;
gfx         PROC

            DECLE       $0000, $0000, $0000, $0000
            DECLE       $0202, $0202, $0202, $0202
            DECLE       $0A0A, $0A0A, $0A0A, $0A0A
            DECLE       $2A2A, $2A2A, $2A2A, $2A2A
            DECLE       $AAAA, $AAAA, $AAAA, $AAAA
            DECLE       $0000, $0000, $0000, $0000
            DECLE       $8080, $8080, $8080, $8080
            DECLE       $A0A0, $A0A0, $A0A0, $A0A0
            DECLE       $A8A8, $A8A8, $A8A8, $A8A8
            DECLE       $AAAA, $AAAA, $AAAA, $AAAA

            ENDP

; -------------------------------------------------------------------------- ;
; External data and routines                                                 ;
; -------------------------------------------------------------------------- ;
            INCLUDE     "util.asm"
