; ========================================================================== ;
;                               = YM Player =                                ;
;                     Intellivision ROM by A.Chevallier                      ;
; -------------------------------------------------------------------------- ;
;                        arnauld.chevallier@gmail.com                        ;
; ========================================================================== ;

; -------------------------------------------------------------------------- ;
; [ clear ]                                                                  ;
; -------------------------------------------------------------------------- ;
; Clear memory area                                                          ;
; -------------------------------------------------------------------------- ;
clear       PROC

            MVI@        R5,         R4
            MVI@        R5,         R1

            CLRR        R0

@@loop:     MVO@        R0,         R4
            DECR        R1
            BNEQ        @@loop

            JR          R5

            ENDP

; -------------------------------------------------------------------------- ;
; [ vblank ]                                                                 ;
; -------------------------------------------------------------------------- ;
; VBlank routine                                                             ;
; -------------------------------------------------------------------------- ;
vblank      PROC

            MVO         R0,         $0020       ; enable display

            MVI         FRAME,      R0          ; frame++
            INCR        R0
            MVO         R0,         FRAME

            MVI         VBTASK,     R7

            ENDP

; -------------------------------------------------------------------------- ;
; [ wait_vbl ]                                                               ;
; -------------------------------------------------------------------------- ;
; Wait for next VBlank interrupt                                             ;
; -------------------------------------------------------------------------- ;
wait_vbl    PROC

            MVI         FRAME,      R0

@@spin:     CMP         FRAME,      R0
            BEQ         @@spin

            JR          R5
            ENDP

; -------------------------------------------------------------------------- ;
; [ grom_text_p ]                                                            ;
; -------------------------------------------------------------------------- ;
; Call to grom_text with inline parameters                                   ;
; -------------------------------------------------------------------------- ;
grom_text_p PROC

            MVI@        R5,         R2
@@ptr:      MVI@        R5,         R4
@@bt:       MVI@        R5,         R3

            ENDP

; -------------------------------------------------------------------------- ;
; [ grom_text ]                                                              ;
; -------------------------------------------------------------------------- ;
; Draw text stored at R4 to output buffer at R3, with color R2               ;
; -------------------------------------------------------------------------- ;
grom_text   PROC
            PSHR        R5

            MOVR        R3,         R5

@@read:     MVI@        R4,         R0          ; read 2 chars
            MOVR        R0,         R1

            ANDI        #$FF,       R0          ; process 1st char
            DECR        R0
            BMI         @@done

            SLL         R0,         2
            SLL         R0
            ADDR        R2,         R0
            MVO@        R0,         R5

            SWAP        R1                      ; process 2nd char
            ANDI        #$FF,       R1
            DECR        R1
            BMI         @@done

            SLL         R1,         2
            SLL         R1
            ADDR        R2,         R1
            MVO@        R1,         R5
            B           @@read

@@done:     PULR        R7
            ENDP

; -------------------------------------------------------------------------- ;
; [ draw_hex ]                                                               ;
; -------------------------------------------------------------------------- ;
; Draw hexadecimal value                                                     ;
; -------------------------------------------------------------------------- ;
draw_hex    PROC

            MVI@        R5,         R2          ; R2 = nb of chars
            MVI@        R5,         R3          ; R3 = pointer to output buffer

@@draw:     MOVR        R0,         R1
            SLR         R0,         2
            SLR         R0,         2
            ANDI        #$F,        R1
            ADDI        #@@chr,     R1
            MVI@        R1,         R1
            MVO@        R1,         R3
            DECR        R3
            DECR        R2
            BNEQ        @@draw

            JR          R5

@@chr:      DECLE       $085, $08D, $095, $09D, $0A5, $0AD, $0B5, $0BD
            DECLE       $0C5, $0CD, $10D, $115, $11D, $125, $12D, $135

            ENDP

; -------------------------------------------------------------------------- ;
; [ hex2dec ]                                                                ;
; -------------------------------------------------------------------------- ;
; Hexa to decimal conversion                                                 ;
; R1 = value to convert                                                      ;
; -------------------------------------------------------------------------- ;
hex2dec     PROC

            MVI@        R5,         R4          ; R4 = pointer into conv. table
            MVI@        R5,         R3          ; R3 = color
            MVI@        R5,         R2          ; R2 = pointer to output buffer
            PSHR        R5

            MOVR        R2,         R5          ; R5 = pointer to output buffer

@@loop:     MVI@        R4,         R0

            SUBR        R0,         R1
            BNC         @@d0
            SUBR        R0,         R1
            BNC         @@d1
            SUBR        R0,         R1
            BNC         @@d2
            SUBR        R0,         R1
            BNC         @@d3
            SUBR        R0,         R1
            BNC         @@d4
            SUBR        R0,         R1
            BNC         @@d5
            SUBR        R0,         R1
            BNC         @@d6
            SUBR        R0,         R1
            BNC         @@d7
            SUBR        R0,         R1
            BNC         @@d8

            MVII        #9*8,       R2
            B           @@save

@@d1:       MVII        #1*8,       R2
            B           @@upd

@@d2:       MVII        #2*8,       R2
            B           @@upd

@@d3:       MVII        #3*8,       R2
            B           @@upd

@@d4:       MVII        #4*8,       R2
            B           @@upd

@@d5:       MVII        #5*8,       R2
            B           @@upd

@@d6:       MVII        #6*8,       R2
            B           @@upd

@@d7:       MVII        #7*8,       R2
            B           @@upd

@@d8:       MVII        #8*8,       R2
            INCR        R7

@@d0:       CLRR        R2

@@upd:      ADDR        R0,         R1

@@save:     ADDR        R3,         R2          ; save digit
            MVO@        R2,         R5

            CMPI        #@@end,     R4          ; next digit
            BNEQ        @@loop

            SLL         R1,         2           ; save last digit
            SLL         R1
            ADDR        R3,         R1
            MVO@        R1,         R5

            PULR        R7

@@5:        DECLE       10000
@@4:        DECLE       1000
@@3:        DECLE       100
@@2:        DECLE       10
@@end:
            ENDP
