<?php
  /* ======================================================================== */
  /*                               [ ymToIntv ]                               */
  /* ======================================================================== */
  /*  This program is free software: you can redistribute it and/or modify    */
  /*  it under the terms of the GNU General Public License as published by    */
  /*  the Free Software Foundation, either version 3 of the License, or       */
  /*  (at your option) any later version.                                     */
  /*                                                                          */
  /*  This program is distributed in the hope that it will be useful,         */
  /*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
  /*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
  /*  GNU General Public License for more details.                            */
  /*                                                                          */
  /*  You should have received a copy of the GNU General Public License       */
  /*  along with this program.  If not, see <http://www.gnu.org/licenses/>.   */
  /* ======================================================================== */
  /*                Copyright (c)2011-2017, Arnauld Chevallier                */
  /*                       arnauld.chevallier@gmail.com                       */
  /* ======================================================================== */
  /*  LHA unpacker:                                                           */
  /*    lhasa by fragglet                                                     */
  /*    License: ISC                                                          */
  /*    Home: http://fragglet.github.io/lhasa/                                */
  /* ======================================================================== */

  /* ======================================================================== */
  /*  Class: ymToIntv                                                         */
  /* ======================================================================== */
  class ymToIntv {
    var $pathToLha;
    var $ymDir;
    var $asmDir;
    var $tmpDir;
    var $romDir;
    var $buf;
    var $packedTmp;
    var $packedStream;
    var $ptr;
    var $hOutput;

    /* ====================================================================== */
    /*  constructor                                                           */
    /* ====================================================================== */
    function ymToIntv($pathToLha, $ymDir, $asmDir, $tmpDir, $romDir) {
      $this->pathToLha = $pathToLha;
      $this->ymDir = $ymDir;
      $this->asmDir = $asmDir;
      $this->tmpDir = $tmpDir;
      $this->romDir = $romDir;
    }

    /* ====================================================================== */
    /*  process()                                                             */
    /* ====================================================================== */
    function process($ymFile, $title = 'untitled', $author = 'unknown', $genRom = false) {
      printf("Processing '%s'\n", $ymFile);

      $asmFile = pathinfo($ymFile)['filename'].'.asm';

      if(($rawFile = $this->unpack($ymFile)) === false) {
        return false;
      }

      $res = $this->parse($rawFile, $asmFile, $title, $author);
      unlink($rawFile);

      if($res && $genRom) {
        $romFile = pathinfo($ymFile)['filename'].'.rom';
        $this->generateRom($asmFile, $romFile);
      }

      printf("\n");

      return $res;
    }

    /* ====================================================================== */
    /*  generateRom()                                                         */
    /* ====================================================================== */
    function generateRom($asmFile, $romFile) {
      printf("Generating '%s'\n", $romFile);
      exec("cd ".$this->asmDir." && as1600 -o ".$this->romDir."/".$romFile." ".$this->asmDir."/data/".$asmFile);
    }

    /* ====================================================================== */
    /*  unpack()                                                              */
    /* ====================================================================== */
    function unpack($ymFile) {
      $output = exec($this->pathToLha.' -xq1w="'.$this->tmpDir.'" "'.$this->ymDir.'/'.$ymFile.'"');
      $output = explode(chr(13), $output);
      $output = array_pop($output);

      if(preg_match("/(.*)\s+-\s+[a-z]+$/i", $output, $match) === 1) {
        return $match[1];
      }
      return false;
    }

    /* ====================================================================== */
    /*  parse()                                                               */
    /* ====================================================================== */
    function parse($inputFile, $outputFile, $ymTitle, $ymAuthor) {
      $this->buf = file_get_contents($inputFile);
      $this->ptr = 0;
      $this->packedTmp = "";
      $this->packedStream = array();

      $id = $this->rd_str(4);

      if($this->rd_str(8) != "LeOnArD!") {
        printf("ERROR - Invalid file header\n");
        return false;
      }

      $frame = $this->rd_lw();
      $attrib = $this->rd_lw();
      $digi = $this->rd_w();
      $clock = $this->rd_lw();
      $freq = $this->rd_w();
      $loop = $this->rd_lw();
      $this->ptr += $this->rd_w();

      for($i = 0; $i < $digi; $i++) {
        $this->ptr += $this->rd_lw();
      }

      $title = $this->rd_nts();
      $author = $this->rd_nts();
      $comment = $this->rd_nts();

      $this->info('Title', $title);
      $this->info('Author', $author);
      $this->info('Comment', $comment);
      $this->info('ID', $id);
      $this->info('Frames', $frame);
      $this->info('Attrib.', $attrib);
      $this->info('Digidrums', $digi);
      $this->info('Clock', $clock);
      $this->info('Freq.', $freq);
      $this->info('Loop', $loop);
      printf("\n");

      $reg = array();

      if($attrib & 1) {
        printf("Loading interleaved data ...\n");

        for($r = 0; $r < 16; $r++) {
          for($f = 0; $f < $frame; $f++) {
            $reg[$f][$r] = $this->rd_b();
          }
        }
      }
      else {
        printf("ERROR - Unsupported file format\n");
        return false;
      }

      $start = 0;
      $end = $frame;

      $totalSize = 0;

      $this->hOutput = fopen($this->asmDir.'/data/'.$outputFile, "w");

      fprintf($this->hOutput, "            INCLUDE     \"player.asm\"\n\n");
      fprintf($this->hOutput, "data        PROC\n\n");
      fprintf($this->hOutput, "@@name      STRING16    \"%20s\"\n", $ymTitle);
      fprintf($this->hOutput, "@@author    STRING16    \"%20s\"\n", $ymAuthor);
      fprintf($this->hOutput, "@@info      DECLE       %d, %d, %d\n\n", $frame, floor($frame / 3000), floor($frame / 50) % 60);

      for($i = 0; $i < 3; $i++) {
        // channel periods
        $p = array();
        for($f = 0; $f < $frame; $f++) {
          $p[$f] = $reg[$f][$i + 8] ? (($reg[$f][$i * 2 + 1] & 0xF) << 8) | $reg[$f][$i * 2] : 0;
        }
        $totalSize += $this->packStream("p".$i, $p, 12, 10, 8, 5);

        // volumes
        $p = array();
        for($f = 0; $f < $frame; $f++) {
          $p[$f] = $reg[$f][$i + 8];
        }
        $totalSize += $this->packStream("v".$i, $p, 5, 10, 8, 5);
      }

      // envelope period
      $p = array();
      for($f = 0; $f < $frame; $f++) {
        $p[$f] = ($reg[$f][12] << 8) | $reg[$f][11];
      }
      $totalSize += $this->packStream("ep", $p, 16, 10, 8, 5);

      // envelope type
      $p = array();
      for($f = 0; $f < $frame; $f++) {
        $p[$f] = $reg[$f][13] == 255 ? 0 : ($reg[$f][13] == 0 ? 1 : $reg[$f][13] & 0xF);
      }
      $totalSize += $this->packStream("et", $p, 4, 10, 8, 5);

      // noise period
      $p = array();
      for($f = 0; $f < $frame; $f++) {
        $p[$f] = $reg[$f][6] & 0x1F;
      }
      $totalSize += $this->packStream("np", $p, 5, 10, 8, 5);

      // mixer
      $p = array();
      for($f = 0; $f < $frame; $f++) {
        $p[$f] = $reg[$f][7] & 0x3F;
      }
      $totalSize += $this->packStream("mx", $p, 6, 10, 8, 5);

      fprintf($this->hOutput, "            ENDP\n\n");

      if($this->rd_str(4) != "End!") {
        printf("ERROR - End marker not found\n");
        return false;
      }

      printf("Success\n\nTotal size = %d bits (%d bytes)\n", $totalSize, ceil($totalSize / 8));

      return true;
    }

    /* ====================================================================== */
    /*  packStream()                                                          */
    /* ====================================================================== */
    function packStream($label, $stream, $dataW, $refW, $refLenW, $blkLenW, &$packedStream = null) {
      printf("Packing @%s stream ...\n", $label);

      $this->packedTmp = "";
      $this->packedStream = array();
      $unpacked = array();
      $size = 0;

      $cnt = count($stream);

      for($p = 0; $p < $cnt;) {
        for($p0 = $p - 1, $bestLen = 0; $p0 >= max(0, $p - (1 << $refW)); $p0--) {
          for($i = 0; $i < (1 << $refLenW) && $p + $i < $cnt && $stream[$p0 + $i] == $stream[$p + $i]; $i++);
          if($i > $bestLen) {
            $bestLen = $i;
            $bestPos = $p0;
          }
        }
        if($refLenW + $refW < $bestLen * $dataW) {
          if($unpacked) {
            $this->toPackedStream(0, 1);
            $this->toPackedStream(count($unpacked) - 1, $blkLenW);
            foreach($unpacked as $d) {
              $this->toPackedStream($d, $dataW);
            }
            $size += 1 + $blkLenW + count($unpacked) * $dataW;
            $unpacked = array();
          }
          $this->toPackedStream(1, 1);
          $this->toPackedStream($p - $bestPos - 1, $refW);
          $this->toPackedStream($bestLen - 1, $refLenW);
          $size += 1 + $refW + $refLenW;
          $p += $bestLen;
        }
        else {
          if(count($unpacked) == (1 << $blkLenW)) {
            $this->toPackedStream(0, 1);
            $this->toPackedStream(count($unpacked) - 1, $blkLenW);
            foreach($unpacked as $d) {
              $this->toPackedStream($d, $dataW);
            }
            $size += 1 + $blkLenW + count($unpacked) * $dataW;
            $unpacked = array();
          }
          $unpacked[] = $stream[$p++];
        }
      }

      if($unpacked) {
        $this->toPackedStream(0, 1);
        $this->toPackedStream(count($unpacked) - 1, $blkLenW);
        foreach($unpacked as $d) {
          $this->toPackedStream($d, $dataW);
        }
        $size += 1 + $blkLenW + count($unpacked) * $dataW;
      }

      $this->toPackedStream(0, 15);

      foreach($this->packedStream as $n => $d) {
        fprintf(
          $this->hOutput,
          "%s$%04X%s",
          !($n % 8) ? str_pad($n ? "" : "@@".$label, 12)."DECLE       " : ", ",
          $d,
          ($n % 8) == 7 ? "\n" : ""
        );
      }
      fprintf(
        $this->hOutput,
        (($n % 8) != 7 ? "\n" : "")."\n"
      );

      return $size;
    }

    /* ====================================================================== */
    /*  toPackedStream()                                                      */
    /* ====================================================================== */
    function toPackedStream($data, $len) {
      $this->packedTmp = strrev(str_pad(decbin($data & ((1 << $len) - 1)), $len, "0", STR_PAD_LEFT)).$this->packedTmp;

      if(strlen($this->packedTmp) >= 16) {
        $this->packedStream[] = bindec(substr($this->packedTmp, -16, 16));
        $this->packedTmp = substr($this->packedTmp, 0, strlen($this->packedTmp) - 16);
      }
    }

    /* ====================================================================== */
    /*  info()                                                                */
    /* ====================================================================== */
    function info($title, $value) {
      printf("%-9s: %s\n", $title, $value);
    }

    /* ====================================================================== */
    /*  rd_nts()                                                              */
    /* ====================================================================== */
    function rd_nts() {
      $str = "";
      while(ord($c = $this->buf[$this->ptr++])) {
        $str .= $c;
      }
      return $str;
    }

    /* ====================================================================== */
    /*  rd_str()                                                              */
    /* ====================================================================== */
    function rd_str($n) {
      $str = substr($this->buf, $this->ptr, $n);
      $this->ptr += $n;
      return $str;
    }

    /* ====================================================================== */
    /*  rd_b()                                                                */
    /* ====================================================================== */
    function rd_b() {
      return ord($this->buf[$this->ptr++]);
    }

    /* ====================================================================== */
    /*  rd_w()                                                                */
    /* ====================================================================== */
    function rd_w() {
      $b0 = $this->rd_b();
      $b1 = $this->rd_b();
      return ($b0 << 8) | $b1;
    }

    /* ====================================================================== */
    /*  rd_lw()                                                               */
    /* ====================================================================== */
    function rd_lw() {
      $w0 = $this->rd_w();
      $w1 = $this->rd_w();
      return ($w0 << 16) | $w1;
    }
  }
?>
